package vc.thinker.absctacts.mqtt.connection;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import vc.thinker.absctacts.mqtt.protocol.IMqttProtocol;

/**
 * Mqtt 回调实现
 *
 * @author HeTongHao
 * @since 2019-10-29 16:00
 */
@Slf4j
public class MqttCallbackImpl implements MqttCallback {
    public MqttCallbackImpl(MqttConnectionProperties connectionProperties, IMqttProtocol mqttProtocol) throws MqttException {
        this.connectionProperties = connectionProperties;
        this.mqttProtocol = mqttProtocol;
        this.reconnectionInterval = connectionProperties.getReconnectionInterval();
        initMqttClient();
    }

    private final MqttConnectionProperties connectionProperties;
    private final IMqttProtocol mqttProtocol;
    private MqttClient mqttClient;

    /**
     * 重连时间，秒
     */
    private int reconnectionInterval;

    @Override
    public void connectionLost(Throwable throwable) {
        log.error("mqtt connectionLost", throwable);
        log.info("mqtt 连接异常，{}稍后重新连接", reconnectionInterval);
        try {
            Thread.sleep(reconnectionInterval * 1000L);
        } catch (InterruptedException e) {
            log.info("", e);
        }
        if (mqttClient != null && !mqttClient.isConnected()) {
            try {
                mqttClient.close();
            } catch (MqttException e) {
                e.printStackTrace();
            }
            try {
                log.info("mqtt 重新连接");
                initMqttClient();
            } catch (Exception e) {
                if (reconnectionInterval < connectionProperties.getReconnectionIntervalMax()) {
                    reconnectionInterval += connectionProperties.getReconnectionIncreasing();
                }
                connectionLost(e);
            }
        }
    }

    @Override
    public void messageArrived(String topic, MqttMessage mqttMessage) {
        if (mqttMessage.isRetained()) {
            return;
        }
        String q = new String(mqttMessage.getPayload());
        if (isClientStatus(topic)) {
            log.info("topic:{},payload:{}", topic, q);
            clientConnected(q);
        } else {
            mqttProtocol.handleMessage(topic, mqttMessage.getPayload());
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
    }


    private MqttClient newMqttClient() throws MqttException {
        MemoryPersistence persistence = new MemoryPersistence();
        MqttClient mqttClient = new MqttClient(connectionProperties.getBroker()
                , connectionProperties.getClientId() + "-" + System.currentTimeMillis(), persistence);
        mqttClient.setCallback(this);
        return mqttClient;
    }

    private void initMqttClient() throws MqttException {
        this.mqttClient = newMqttClient();
        mqttProtocol.setMqttClient(mqttClient);
        connect(mqttClient, mqttProtocol.subscribe());
    }

    /**
     * 连接配制
     *
     * @return
     */
    private MqttConnectOptions getConnOpts() {
        MqttConnectOptions connOpts = new MqttConnectOptions();
        connOpts.setCleanSession(false);
        connOpts.setUserName(connectionProperties.getUsername());
        connOpts.setPassword(connectionProperties.getPassword().toCharArray());
        try {
            // ssl
//            connOpts.setSocketFactory(SslUtil.getSocketFactory(
//                    ResourceUtils.getURL("classpath:").getPath() + certFile));
        } catch (Exception e) {
            e.printStackTrace();
        }
        connOpts.setKeepAliveInterval(30);
        connOpts.setConnectionTimeout(60 * 5);
        connOpts.setCleanSession(true);
        connOpts.setAutomaticReconnect(true);
        return connOpts;
    }

    /**
     * 判断连接状态
     *
     * @param topic
     * @return
     */
    private boolean isClientStatus(String topic) {
        return topic.endsWith("connected");
    }

    /**
     * 连接
     *
     * @throws MqttSecurityException
     * @throws MqttException
     */
    private void connect(MqttClient mqttClient, String[] topic) throws MqttSecurityException, MqttException {
        mqttClient.connect(getConnOpts());
        mqttClient.subscribe(topic);
        log.info("mqtt client [{}] connect [{}] Success"
                , connectionProperties.getClientId(), connectionProperties.getBroker());
    }

    /**
     * 客户端连接状态处理
     *
     * @param q
     */
    private void clientConnected(String q) {
        JSONObject session = JSONObject.parseObject(q);
        String clientId = session.getString("clientid");
        if (session.containsKey("ipaddress")) {
            log.info("{} 连接服务器", clientId);
        } else {
            log.info("{} 断开服务器", clientId);
        }
    }
}
